class ArchiveError(Exception): ...
class Bad7zFile(ArchiveError): ...
class CrcError(ArchiveError): ...
class UnsupportedCompressionMethodError(ArchiveError): ...
class DecompressionError(ArchiveError): ...
class InternalError(ArchiveError): ...
class PasswordRequired(Exception): ...
