from typing import Any

MAGIC_7Z: Any
FINISH_7Z: Any
COMMAND_HELP_STRING: str

def get_default_blocksize(): ...

FILTER_LZMA: Any
FILTER_LZMA2: Any
FILTER_DELTA: Any
FILTER_ARM: Any
FILTER_ARMTHUMB: Any
FILTER_IA64: Any
FILTER_POWERPC: Any
FILTER_SPARC: Any
FILTER_X86: Any
CHECK_CRC32: Any
CHECK_CRC64: Any
CHECK_SHA256: Any
CHECK_NONE: Any
CHECK_ID_MAX: Any
CHECK_UNKNOWN: Any
PRESET_DEFAULT: Any
PRESET_EXTREME: Any
FILTER_CRYPTO_AES256_SHA256: int
FILTER_CRYPTO_ZIP: int
FILTER_CRYPTO_RAR29: int
FILTER_BZIP2: int
FILTER_DEFLATE: int
FILTER_COPY: int
FILTER_ZSTD: int
FILTER_PPMD: int
FILTER_BROTLI: int

class Constant:
    def __setattr__(self, *_: Any) -> None: ...

class DefaultFilters(Constant):
    ARCHIVE_FILTER: Any = ...
    ENCODED_HEADER_FILTER: Any = ...
    ENCRYPTED_ARCHIVE_FILTER: Any = ...
    ENCRYPTED_HEADER_FILTER: Any = ...

DEFAULT_FILTERS: Any

class Property(Constant):
    END: Any = ...
    HEADER: Any = ...
    ARCHIVE_PROPERTIES: Any = ...
    ADDITIONAL_STREAMS_INFO: Any = ...
    MAIN_STREAMS_INFO: Any = ...
    FILES_INFO: Any = ...
    PACK_INFO: Any = ...
    UNPACK_INFO: Any = ...
    SUBSTREAMS_INFO: Any = ...
    SIZE: Any = ...
    CRC: Any = ...
    FOLDER: Any = ...
    CODERS_UNPACK_SIZE: Any = ...
    NUM_UNPACK_STREAM: Any = ...
    EMPTY_STREAM: Any = ...
    EMPTY_FILE: Any = ...
    ANTI: Any = ...
    NAME: Any = ...
    CREATION_TIME: Any = ...
    LAST_ACCESS_TIME: Any = ...
    LAST_WRITE_TIME: Any = ...
    ATTRIBUTES: Any = ...
    COMMENT: Any = ...
    ENCODED_HEADER: Any = ...
    START_POS: Any = ...
    DUMMY: Any = ...

PROPERTY: Any

class CompressionMethod(Constant):
    COPY: Any = ...
    DELTA: Any = ...
    BCJ: Any = ...
    PPC: Any = ...
    IA64: Any = ...
    ARM: Any = ...
    ARMT: Any = ...
    SPARC: Any = ...
    SWAP2: Any = ...
    SWAP4: Any = ...
    LZMA: Any = ...
    PPMD: Any = ...
    P7Z_BCJ: Any = ...
    P7Z_BCJ2: Any = ...
    BCJ_PPC: Any = ...
    BCJ_IA64: Any = ...
    BCJ_ARM: Any = ...
    BCJ_ARMT: Any = ...
    BCJ_SPARC: Any = ...
    LZMA2: Any = ...
    MISC_ZIP: Any = ...
    MISC_BZIP2: Any = ...
    MISC_DEFLATE: Any = ...
    MISC_DEFLATE64: Any = ...
    MISC_Z: Any = ...
    MISC_LZH: Any = ...
    NSIS_DEFLATE: Any = ...
    NSIS_BZIP2: Any = ...
    MISC_ZSTD: Any = ...
    MISC_BROTLI: Any = ...
    MISC_LZ4: Any = ...
    MISC_LZS: Any = ...
    MISC_LIZARD: Any = ...
    CRYPT_ZIPCRYPT: Any = ...
    CRYPT_RAR29AES: Any = ...
    CRYPT_AES256_SHA256: Any = ...

COMPRESSION_METHOD: Any
